#!/bin/bash

wait_for_k3s() {
    local max_wait_secs=30
    local start_time=$(date +%s)

    while true; do
        if systemctl is-active k3s --quiet; then
            echo "K3s is running."
            break
        elif [ $(($(date +%s) - start_time)) -lt $max_wait_secs ]; then
            echo "K3s activating, waiting..."
            sleep 5
        else
            echo "K3s failed to start, aborting."
            exit 1
        fi
    done
}

wait_for_kpec() {
    local max_wait_secs=300
    local start_time=$(date +%s)
    while true; do
        if systemctl is-active kpec --quiet; then
            echo "KMS Plugin for Edge Clusters is running."
            break
        elif [ $(($(date +%s) - start_time)) -lt $max_wait_secs ]; then
            echo "KMS Plugin for Edge Clusters activating, waiting..."
            sleep 5
        else
            echo "KMS Plugin for Edge Clusters failed to start, aborting."
            exit 1
        fi
    done
}

verify_control_plane_node() {
    export HOSTNAME="$(uname -n)"
    HOSTNAME="${HOSTNAME,,}"

    result=$(kubectl describe nodes $HOSTNAME)
    if [ $? -ne 0 ]; then
        echo "$result"
        exit 1
    fi

    if ! echo "$result" | grep -q "Roles.*control-plane"; then
        echo "Error: KMS Plugin for Edge Clusters must be installed on a control-plane node."
    fi
}

check_if_root() {
    if [ $EUID -ne 0 ]; then
        echo "This script must be run as root, exiting."
        exit 1
    fi
}

rotate_secrets() {
    attempt=0
    max_attempts=2

    /usr/local/bin/kubectl get secrets --all-namespaces -o json | /usr/local/bin/kubectl replace -f -

    result=$?

    while [ $result -ne 0 ] && [ $attempt -lt $max_attempts ]; do
        echo "Retrying secret rotation"
        sleep 10

        /usr/local/bin/kubectl get secrets --all-namespaces -o json | /usr/local/bin/kubectl replace -f -
        result=$?
        attempt=$((attempt + 1))
    done
}

usage() {
    echo "Usage: $0 <function> [arguments]"
    echo "Available functions:"
    echo "  wait_for_k3s  - Wait for k3s systemd service to show as 'active'."
    echo "  wait_for_kpec  - Wait for KPEC systemd service to show as 'active'."
    echo "  verify_control_plane_node  - Check if the current cluster has the 'control-plane' node role."
    echo "  check_if_root  - Check if the script is being run as root."
    echo "  rotate_secrets  - Re-encrypt cluster secrets."

    exit 1
}

if [[ $# -lt 1 ]]; then
    usage
fi

FUNCTION=$1
shift # Shift arguments to remove the function name from the list

# Call the corresponding function
case "$FUNCTION" in
    wait_for_k3s)
        wait_for_k3s
        ;;
    wait_for_kpec)
        wait_for_kpec
        ;;
    verify_control_plane_node)
        verify_control_plane_node
        ;;
    check_if_root)
        check_if_root
        ;;
    rotate_secrets)
        rotate_secrets
        ;;   
    *)
        echo "Error: Unknown function '$FUNCTION'"
        usage
        ;;
esac
